@@ -289,11 +289,11 @@ bool TGAImage::set(int x, int y, const TGAColor &c) {
   return true;
 }
 
-int TGAImage::get_bytespp() { return bytespp; }
+int TGAImage::get_bytespp() const { return bytespp; }
 
-int TGAImage::get_width() { return width; }
+int TGAImage::get_width() const { return width; }
 
-int TGAImage::get_height() { return height; }
+int TGAImage::get_height() const { return height; }
 
 bool TGAImage::flip_horizontally() {
   if (!data) return false;
